@@ -1,6 +1,21 @@
+# Copyright 2015 Oliver Cope
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
+#
+from __future__ import absolute_import
+
 import sys
 import inspect
-from json import dumps
 from operator import attrgetter
 from functools import wraps, partial
 from itertools import cycle
@@ -9,6 +24,8 @@ import warnings
 from fresco import GET, POST, PUT, Response
 from fresco.exceptions import BadRequest
 
+_marker = object()
+
 
 def onerror(exceptions, handler):
     """\
@@ -34,16 +51,21 @@ def onerror(exceptions, handler):
     return decorator
 
 
-def json_response(data=None, indent=None, separators=(',', ':'), **kwargs):
+def json_response(data=_marker,
+                  indent=None,
+                  separators=(',', ':'),
+                  content_type='application/json',
+                  **kwargs):
     """
-    JSON encode the given data and return a response object with the json mime
-    type.
+    JSON encode the function's result and return a response object with the
+    content-type ``application/json``.
 
-    Can be used as a function decorator, in which case it will wrap the
-    function's return value as a json response.
+    May also be used as a regular function to directly create a JSON encoded
+    response, however it's preferrable to use
+    :meth:`~fresco.response.Response.json` for this.
 
     :param data: The data to json encode.
-                 If left as None, ``json_response``
+                 If left unspecified, ``json_response``
                  will act as a function decorator.
     :param indent: The indent level. Defaults to ``None`` (no pretty printing)
     :param separators: Defaults to ``(',', ':')`` for the most compact JSON
@@ -54,22 +76,32 @@ def json_response(data=None, indent=None, separators=(',', ':'), **kwargs):
 
 
     """
-    if data is None:
+    # Called as a decorator or decorator factory
+    if data is _marker or callable(data):
         def json_response_decorator(func):
             @wraps(func)
             def json_response_decorated(*fa, **fkw):
-                return json_response(
+                return Response.json(
                     func(*fa, **fkw), indent, separators, **kwargs)
             return json_response_decorated
-        return json_response_decorator
 
-    if isinstance(data, Response):
-        return data
+        # Called as a decorator factory (``@json_response()``)
+        if data is _marker:
+            return json_response_decorator
 
-    return Response(
-            content_type='application/json',
-            content=[dumps(
-                data, indent=indent, separators=separators, **kwargs)])
+        # Called as a decorator (ie no parens: ``@json_response``)
+        return json_response_decorator(data)
+
+    # Called as a regular function (or via ``Route(...).filter``)
+    else:
+        if isinstance(data, Response):
+            return data
+
+        return Response.json(data,
+                            indent=indent,
+                            separators=separators,
+                            content_type=content_type,
+                            **kwargs)
 
 
 def extract_requestargs(requesttype, **argspec):
