@@ -1,3 +1,19 @@
+# Copyright 2015 Oliver Cope
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
+#
+from __future__ import absolute_import
+
 from collections import defaultdict
 from fresco.compat import get_ident
 
@@ -13,6 +29,14 @@ class RequestContext(object):
     give a different value per request. In a threaded environment this would be
     ``thread.get_ident``. If using a different execution model, a different
     strategy would need to be found.
+
+    During a request the context object will contain the following keys:
+
+    - ``app``: the currently executing FrescoApp object.
+    - ``route_traversal``: a :class:`~fresco.routing.RouteTraversal` object.
+    - ``view_self``: the instance of the current class based view if
+      applicable, otherwise ``None``.
+    - ``request``: the current :class:`~fresco.request.Request` object.
     """
 
     def __init__(self, ident_func=get_ident):
