@@ -1,13 +1,30 @@
+# Copyright 2015 Oliver Cope
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
+#
 """
 The :class:`Response` class models the response from your application to a
 single request.
 """
+from __future__ import absolute_import
 
 import re
+import json
 from itertools import chain
 
 import fresco
 from .cookie import Cookie
+from .compat import ustr
 
 __all__ = [
     'STATUS_CONTINUE', 'STATUS_SWITCHING_PROTOCOLS', 'STATUS_PROCESSING',
@@ -200,13 +217,15 @@ HEADER_NAMES = {
     'x_ua_compatible': 'X-UA-Compatible',
 }
 
+default_charset = 'UTF-8'
+
 
 def encoder(stream, charset):
     r"""\
     Encode a response iterator using the given character set.
     """
     if charset is None:
-        charset = 'UTF-8'
+        charset = default_charset
 
     for chunk in stream:
         if not isinstance(chunk, bytes):
@@ -222,8 +241,8 @@ class Response(object):
 
     default_content_type = "text/html; charset=UTF-8"
 
-    def __init__(self, content=None, status="200 OK", headers=None,
-                 onclose=None, _nocontent=[], **kwargs):
+    def __init__(self, content=None, status=None, headers=None,
+                 onclose=None, _nocontent=[], passthrough=False, **kwargs):
         """
         Create a new Response object, modelling the HTTP status, headers and
         content of an HTTP response. Response instances are valid WSGI
@@ -232,6 +251,10 @@ class Response(object):
         :param content: The response content as an iterable object
         :param status: The HTTP status line, eg ``200 OK`` or ``404 Not Found``
         :param headers: A list of HTTP headers
+        :param passthrough: If True, use the content iterator unmodified.
+                            Default behaviour is to wrap the content iterator
+                            with :func:`encoder` to encode unicode strings
+                            before output.
         :param kwargs: Arbitrary headers, provided as keyword arguments.
                        Underscores will be replaced with hyphens (eg
                        ``content_length`` becomes ``Content-Length``).
@@ -253,12 +276,15 @@ class Response(object):
             ...               content_type='text/html')
 
         """
-
         if content is None:
             content = _nocontent
+            if status is None:
+                status = '204 No Content'
 
-        self._content = content
-        self._status = make_status(status)
+        if status is None:
+            status = '200 OK'
+
+        self.status = make_status(status)
         if onclose is None:
             self.onclose = []
         elif callable(onclose):
@@ -279,35 +305,55 @@ class Response(object):
             else:
                 headers.insert(0, ('Content-Type', self.default_content_type))
 
-        self._headers = headers
+        self.headers = headers
+
+        # Now we've dealt with the headers (including Content-Type,
+        # which the charset property will look for when deciding how to
+        # encode strings) we can pull the content into an iterable.
+        # We optimize for the common cases (byte string, unicode string,
+        # list of byte strings, list of unicode strings), otherwise
+        # we wrap the iterator in :func:`encoder`, which iterates the content
+        # and tries to encode each item as a byte string. Note that this
+        # code assumes that lists are homogeneous.
+        self.content = content
+        content_islist = isinstance(content, list)
+        if passthrough:
+            self.content_iterator = content
+        elif content_islist and len(content) == 0:
+            self.content_iterator = content
+        elif content_islist and isinstance(content[0], bytes):
+            self.content_iterator = content
+        elif content_islist and isinstance(content[0], ustr):
+            charset = self.charset
+            self.content_iterator = [c.encode(charset) for c in content]
+        elif isinstance(content, bytes):
+            self.content_iterator = [content]
+        elif isinstance(content, ustr):
+            self.content_iterator = [content.encode(self.charset)]
+        else:
+            self.content_iterator = _copy_close(
+                content, encoder(content, self.charset))
 
     def __call__(self, environ, start_response, exc_info=None):
         """
         WSGI callable. Calls ``start_response`` with assigned headers and
         returns an iterator over ``content``.
         """
-        start_response(
-            self.status,
-            self.headers,
-            exc_info,
-        )
-        result = _copy_close(self.content, encoder(self.content, self.charset))
+        start_response(self.status, self.headers, exc_info)
+        result = self.content_iterator
         if self.onclose:
             result = ClosingIterator(result, *self.onclose)
         return result
 
     def add_onclose(self, *funcs):
         """
-        Add functions to be called as part of the response iterators ``close``
+        Add functions to be called as part of the response iterator's ``close``
         method.
         """
-        return self.__class__(
-            self.content,
-            self.status,
-            self.headers,
-            self.onclose + list(funcs),
-            add_default_content_type=False
-        )
+        return self.__class__(self.content,
+                              self.status,
+                              self.headers,
+                              self.onclose + list(funcs))
 
     @classmethod
     def from_wsgi(cls, wsgi_callable, environ, start_response):
@@ -322,41 +368,22 @@ class Response(object):
             content = _copy_close(content,
                                   chain(content, [responder.buf.getvalue()]))
 
-        if responder.called:
-            return cls(content, responder.status, headers=responder.headers)
-
-        # Iterator has not called start_response yet. Call content.next()
-        # to force the application to call start_response
-        try:
-            chunk = content.next()
-        except StopIteration:
-            return cls(content, responder.status, headers=responder.headers)
-        except Exception:
-            close = getattr(content, 'close', None)
-            if close is not None:
-                close()
-            raise
-        content = _copy_close(content, chain([chunk], content))
-        return cls(
-            content,
-            responder.status,
-            headers=responder.headers
-        )
-
-    @property
-    def content(self):
-        """
-        Iterator over the response content part
-        """
-        return self._content
-
-    @property
-    def headers(self):
-        """\
-        Return a list of response headers in the format
-        ``[(<header-name>, <value>), ...]``
-        """
-        return self._headers
+        if not responder.called:
+            # Iterator has not called start_response yet. Call next(content)
+            # to force the application to call start_response
+            try:
+                chunk = next(content)
+            except StopIteration:
+                pass
+            except Exception:
+                close = getattr(content, 'close', None)
+                if close is not None:
+                    close()
+                raise
+            else:
+                content = _copy_close(content, chain([chunk], content))
+        return cls(content, responder.status, headers=responder.headers,
+                   passthrough=True)
 
     def get_headers(self, name):
         """\
@@ -391,16 +418,9 @@ class Response(object):
             return default
         return ','.join(headers)
 
-    @property
-    def status(self):
-        """\
-        HTTP status message for the response, eg ``200 OK``
-        """
-        return self._status
-
     @property
     def status_code(self):
-        """\
+        """
         Return the numeric status code for the response as an integer::
 
             >>> Response(status='404 Not Found').status_code
@@ -408,7 +428,7 @@ class Response(object):
             >>> Response(status=200).status_code
             200
         """
-        return int(self._status.split(' ', 1)[0])
+        return int(self.status.split(' ', 1)[0])
 
     @property
     def content_type(self):
@@ -433,8 +453,7 @@ class Response(object):
             >>> r.add_header('Cache-Control', 'no-cache').headers
             [('Content-Type', 'text/plain'), ('Cache-Control', 'no-cache')]
         """
-        return self.replace(self._content, self._status,
-                            self._headers + [(name, value)])
+        return self.replace(headers=self.headers + [(name, value)])
 
     def add_headers(self, headers=[], **kwheaders):
         """\
@@ -450,12 +469,12 @@ class Response(object):
             ... ).headers
             [('Content-Type', 'text/plain'), ('Cache-Control', 'no-cache')]
         """
-        return self.replace(headers=make_headers(self._headers + headers,
+        return self.replace(headers=make_headers(self.headers + headers,
                                                       kwheaders))
 
     def remove_headers(self, *headers):
         """\
-        Return a new response object with the given headers removed.
+        Return a new response object with the named headers removed.
 
         Synopsis::
 
@@ -467,13 +486,11 @@ class Response(object):
             [('Content-Type', 'text/plain')]
         """
         toremove = [item.lower() for item in headers]
-        return self.replace(headers=[h for h in self._headers
+        return self.replace(headers=[h for h in self.headers
                                        if h[0].lower() not in toremove])
 
-    def add_cookie(
-        self, name, value, maxage=None, expires=None, path=None,
-        secure=None, domain=None, comment=None, http_only=False, version=1
-    ):
+    def add_cookie(self, name, value, max_age=None, expires=None, path='/',
+                   secure=None, domain=None, comment=None, httponly=False):
         """\
         Return a new response object with the given cookie added.
 
@@ -485,20 +502,36 @@ class Response(object):
             >>> r.add_cookie('a', '1').headers
             [('Content-Type', 'text/plain'), ('Set-Cookie', 'a=1;Version=1')]
         """
+        return self.add_header('Set-Cookie', Cookie(name, value, max_age,
+                                                    expires, path, secure,
+                                                    domain,
+                                                    comment=comment,
+                                                    httponly=httponly))
+
+    def delete_cookie(self, name, path='/', domain=None):
+        return self.add_cookie(name, '', max_age=0, path=path, domain=domain,
+                               expires='Thu, 01-Jan-1970 00:00:00 GMT')
+
+    def add_vary(self, *vary_on):
+        """
+        Return a new response object with the given Vary header.
+        Values specified will be added to any existing vary header.
+
+        Synopsis::
+
+            >>> r = Response().add_vary('Accept-Encoding')
+            >>> r.headers
+            [('Vary', 'Accept-Encoding')]
+        """
+        vary_on = set(vary_on)
+        newheaders = []
+        for k, v in self.headers:
+            if k.lower() == 'vary':
+                vary_on.update(s.strip() for s in v.split(','))
+            else:
+                newheaders.append((k, v))
         return self.replace(
-            headers=self._headers + [
-                (
-                    'Set-Cookie',
-                    Cookie(
-                        name, value, maxage, expires, path,
-                        secure, domain,
-                        comment=comment,
-                        http_only=http_only,
-                        version=version
-                    )
-                )
-            ]
-        )
+            headers=newheaders + [('Vary', ', '.join(vary_on))])
 
     def replace(self, content=None, status=None, headers=None, **kwheaders):
         """\
@@ -514,38 +547,27 @@ class Response(object):
             ...               content_length=3)
 
         """
-
-        res = self
-
-        if content is not None:
+        if content is None:
+            content = self.content
+            onclose = self.onclose
+        else:
             close = getattr(self.content, 'close', None)
             onclose = self.onclose
             if close:
                 onclose = [close] + onclose
-            res = res.__class__(content, res._status, res._headers,
-                                onclose=onclose, content_type=None)
 
-        if headers is not None:
-            res = res.__class__(res._content, res._status, headers,
-                                onclose=res.onclose, content_type=None)
+        if headers is None:
+            headers = self.headers
 
-        if status is not None:
-            res = res.__class__(res._content, status, res._headers,
-                                onclose=res.onclose, content_type=None)
+        if status is None:
+            status = self.status
 
         if kwheaders:
             toremove = set(make_header_name(k) for k in kwheaders)
-            kwheaders = make_headers([], kwheaders)
-            res = res.__class__(
-                res._content,
-                res._status,
-                [(key, value) for key, value in res._headers
-                              if key not in toremove] + kwheaders,
-                onclose=res.onclose,
-                add_default_content_type=False
-            )
+            headers = ([(k, v) for k, v in headers if k not in toremove] +
+                       make_headers([], kwheaders))
 
-        return res
+        return self.__class__(content, status, headers, onclose=onclose)
 
     def buffered(self):
         """\
@@ -565,7 +587,7 @@ class Response(object):
             >>> r.content
             ['one two ', 'three four five']
         """
-        content = list(self.content)
+        content = list(self.content_iterator)
         content_length = sum(map(len, content))
         return self.replace(content=content, content_length=content_length)
 
@@ -579,9 +601,8 @@ class Response(object):
                 mo = _parser(val)
                 if mo:
                     return mo.group(1)
-                else:
-                    return None
-        return None
+                break
+        return default_charset
 
     @classmethod
     def not_found(cls, request=None):
@@ -618,14 +639,12 @@ class Response(object):
             ...
         """
         return cls(
+            "<html>\n"
+            "<body>\n"
+            "<h1>%s</h1>\n"
+            "</body>\n"
+            "</html>" % (message,),
             status=STATUS_FORBIDDEN,
-            content=[
-                "<html>\n"
-                "<body>\n"
-                "<h1>" + message + "</h1>\n"
-                "</body>\n"
-                "</html>"
-            ]
         )
 
     @classmethod
@@ -741,11 +760,11 @@ class Response(object):
                      "</html>"])
 
     @classmethod
-    def redirect(cls, location, request=None, status=STATUS_FOUND):
+    def redirect(cls, location, request=None, status=STATUS_FOUND, **kwargs):
         """\
         Return an HTTP redirect reponse (302 or 301).
 
-        :param location: The redirect location.
+        :param location: The redirect location or a view specification.
         :param status: HTTP status code for the redirect, default is
                        ``STATUS_FOUND`` (temporary redirect)
 
@@ -754,21 +773,54 @@ class Response(object):
             >>> def view():
             ...   return Response.redirect("/new-location")
             ...
+
+        The location parameter is interpreted as follows:
+
+        - If it is a callable it is assumed to be a view,
+          and passed to :func:`fresco.core.urlfor` to resolve it to a URL.
+
+        - If it is a string and contains '://' it is assumed to be an absolute
+          URL and no further processing is done.
+
+        - If it is a string and contains a slash,
+          it is assumed to be a
+          relative URL and resolved relative to the current request.
+
+        - If it is any other string
+          it is passed to :func:`fresco.core.urlfor` to resolve it to a URL.
+          If this fails (ie raises RouteNotFound) it is assumed to be a
+          relative URL and resolved relative to the current request.
         """
-        if '://' not in location:
+        if callable(location):
+            location = fresco.context.app.urlfor(location, **kwargs)
+
+        elif '://' in location:
+            pass
+
+        elif '/' in location:
             if request is None:
                 request = fresco.context.request
             location = request.resolve_url(location)
+
+        else:
+            from fresco.routing import RouteNotFound
+            try:
+                location = fresco.context.app.urlfor(location, **kwargs)
+            except RouteNotFound:
+                if kwargs:
+                    # kwargs only makes sense if this is a view spec
+                    raise
+                if request is None:
+                    request = fresco.context.request
+                location = request.resolve_url(location)
+
         return Response(
+            "<html><head></head><body>\n"
+            "<h1>Page has moved</h1>\n"
+            "<p><a href='%s'>%s</a></p>\n"
+            "</body></html>" % (location, location),
             status=status,
-            location=location,
-            content=[
-                "<html><head></head><body>\n",
-                "<h1>Page has moved</h1>\n",
-                "<p><a href='%s'>%s</a></p>\n" % (location, location),
-                "</body></html>",
-            ]
-        )
+            location=location)
 
     @classmethod
     def redirect_permanent(cls, *args, **kwargs):
@@ -818,6 +870,38 @@ class Response(object):
             '</html>').format(location)
         ], content_type='text/html')
 
+    @classmethod
+    def json(cls,
+             data,
+             indent=None,
+             separators=(',', ':'),
+             content_type='application/json',
+             status=None,
+             headers=None,
+             dumps=json.dumps,
+             **kwargs):
+        """
+        Return an ``application/json`` response with the given data
+        JSON serialized
+
+        :param data: The data to json encode.
+        :param indent: The indent level.
+                       Defaults to ``None`` (no pretty printing)
+        :param separators: Defaults to ``(',', ':')`` for the most compact JSON
+                           representation
+        :param kwargs: Other keyword arguments are passed to ``json.dumps``.
+                       These may be used to change encoding paramters, for
+                       example overriding the default ``JSONEncoder`` class.
+
+        """
+        return cls([dumps(data,
+                          indent=indent,
+                          separators=separators,
+                          **kwargs)],
+                   status=status,
+                   headers=headers,
+                   content_type=content_type,)
+
 
 def make_header_name(name):
     """\
@@ -869,19 +953,25 @@ def make_headers(header_list, header_dict):
     return newheaders
 
 
-def dump_response(r, line_break='\r\n'):
+def dump_response(r, line_break=b'\r\n', encoding='UTF-8'):
     """\
-    Return a string representation of the given response, as for a HTTP
+    Return a byte-string representation of the given response, as for a HTTP
     response.
     """
     output = []
-    output.append(r.status)
+    output.append(r.status.encode('ascii'))
     output.append(line_break)
-    output.extend('{}: {}{}'.format(k, v, line_break)
-                  for k, v in sorted(r.headers))
+    for k, v in sorted(r.headers):
+        output.append(k.encode('ascii'))
+        output.append(b': ')
+        output.append(v.encode('ascii'))
+        output.append(line_break)
     output.append(line_break)
-    output.extend(r.content)
-    return ''.join(output)
+    output.extend(r.content_iterator)
+    s = b''.join(output)
+    if encoding:
+        return s.decode(encoding)
+    return s
 
 
 def make_status(status):
