@@ -190,8 +190,8 @@ class RequestArg(RouteArg):
                 value = self.converter(value)
         except self.converter_exceptions:
             value = LazyException(
-                        self.exception('%r is not a valid value for %s' %
-                                       (value, self.formkey,)))
+                        self.exception('Invalid value for %s' %
+                                       (self.formkey,)))
 
         return value
 
