@@ -1036,7 +1036,10 @@ class RouteCollection(object):
                          'package.module.viewfunction'
         """
         try:
-            return self.__routed_views__[viewspec]
+            r = self.__routed_views__[viewspec]
+            if isinstance(r, RouteNotFound):
+                raise r
+            return r
         except KeyError:
             pass
 
@@ -1050,14 +1053,16 @@ class RouteCollection(object):
             pass
 
         if not isinstance(viewspec, string_types):
-            raise RouteNotFound(viewspec)
+            exc = self.__routed_views__[viewspec] = RouteNotFound(viewspec)
+            raise exc
 
         modname, symbols = viewspec, []
         while True:
             try:
                 modname, sym = modname.rsplit('.', 1)
             except ValueError:
-                raise RouteNotFound(viewspec)
+                exc = self.__routed_views__[viewspec] = RouteNotFound(viewspec)
+                raise exc
 
             symbols.append(sym)
             module = sys.modules.get(modname, None)
@@ -1072,7 +1077,9 @@ class RouteCollection(object):
                     try:
                         route = self.__routed_views__[ob.__func__]
                     except (AttributeError, KeyError):
-                        raise RouteNotFound(viewspec)
+                        exc = self.__routed_views__[viewspec] = \
+                                RouteNotFound(viewspec)
+                        raise exc
 
                 self.__routed_views__[viewspec] = route
                 return route
