@@ -1,6 +1,21 @@
+# Copyright 2015 Oliver Cope
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
+#
 """
 Utilities for dealing with static file content
 """
+from __future__ import absolute_import
 
 import mimetypes
 import os
@@ -61,5 +76,5 @@ def serve_static_file(path, default_charset="ISO-8859-1",
         last_modified=formatdate(mtime, localtime=False, usegmt=True),
         content_type=typ,
         content_encoding=enc,
-        content=content_iterator(_file)
-    )
+        content=content_iterator(_file),
+        passthrough=True)
