@@ -1,3 +1,19 @@
+from __future__ import absolute_import
+
+# Copyright 2015 Oliver Cope
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
+#
 import sys
 import os
 import re
@@ -30,7 +46,7 @@ class TestServeStaticFile():
     def test_sets_last_modified_header(self):
         with FrescoApp().requestcontext():
             response = serve_static_file(self.tmpname)
-            response.content.close()
+            response.content_iterator.close()
 
         last_modified = response.get_header('Last-Modified')
         assert self.parse_lmh(last_modified) == self.mtime
@@ -38,7 +54,7 @@ class TestServeStaticFile():
     def test_full_response_when_if_modified_before_mtime(self):
         with FrescoApp().requestcontext(**self.make_ims(0)):
             response = serve_static_file(self.tmpname)
-            response.content.close()
+            response.content_iterator.close()
 
         last_modified = response.get_header('Last-Modified')
         assert response.status_code == 200
@@ -75,7 +91,7 @@ class TestServeStaticFile():
     def test_last_modified_format_is_correct(self):
         with FrescoApp().requestcontext():
             response = serve_static_file(self.tmpname)
-            response.content.close()
+            response.content_iterator.close()
 
         assert re.match(r'^\w{3}, \d{1,2} \w{3} \d{4} \d\d:\d\d:\d\d GMT',
                         response.get_header('last-modified'))
@@ -86,7 +102,7 @@ class TestServeStaticFile():
             c.request.environ['wsgi.file_wrapper'] = file_wrapper
             serve_static_file(self.tmpname)
 
-        file_wrapper.assert_called_once()
+        assert file_wrapper.call_count == 1
 
     def test_it_returns_not_found(self):
         with FrescoApp().requestcontext():
