@@ -1,16 +1,26 @@
-# coding=UTF-8
-# Copyright (c) 2009 Oliver Cope. All rights reserved.
+# encoding=UTF-8
+# Copyright 2015 Oliver Cope
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
+#
 # See LICENSE.txt for terms of redistribution and use.
 
 from __future__ import absolute_import
 from __future__ import unicode_literals
 from io import BytesIO
 
-from flea import Agent
-
-from fresco.compat import quote, ustring_type
+from fresco.compat import quote, ustr
 from fresco.util.http import parse_querystring, parse_post, encode_multipart
-from fresco.util.wsgi import unicode_to_environ
 from fresco import FrescoApp, context, Response, POST
 
 from . import form_data
@@ -85,35 +95,30 @@ class TestParseMultipart(object):
         app = FrescoApp()
         app.route('/', POST, view)
 
-        r = Agent(app).post_multipart(
-            files=[('f1', 'filename.txt', 'text/plain', 'x' * 1000)],
-            check_status=False
-        )
-        assert r.response.status == "413 Payload Too Large"
+        with app.requestcontext_post(
+                '/', files=[('f1', 'filename.txt', 'text/plain', 'x' * 1000)]):
+            assert app.view().status == "413 Payload Too Large"
 
-        r = Agent(app).post_multipart(
-            files=[('f1', 'filename.txt', 'text/plain', 'x' * 400)],
-            data={'f2': 'x' * 101},
-            check_status=False
-        )
-        assert r.response.status == "413 Payload Too Large"
+        with app.requestcontext_post(
+                '/',
+                files=[('f1', 'filename.txt', 'text/plain', 'x' * 400)],
+                data={'f2': 'x' * 101}):
+            assert app.view().status == "413 Payload Too Large"
 
     def test_fileupload_with_invalid_content_length(self):
 
         def view():
             request = context.request
-            request.MAX_MULTIPART_SIZE = 500
             request.get('f1')
             return Response(['ok'])
         app = FrescoApp()
         app.route('/', POST, view)
 
-        r = Agent(app).post_multipart(
-            files=[('f1', 'filename.txt', 'text/plain', 'x' * 1000)],
-            CONTENT_LENGTH=unicode_to_environ("499"),
-            check_status=False
-        )
-        assert r.response.status == "400 Bad Request"
+        with app.requestcontext_post(
+                '/',
+                files=[('f1', 'filename.txt', 'text/plain', 'x' * 1000)]) as c:
+            c.request.environ['CONTENT_LENGTH'] = str('500')
+            assert app.view().status == "400 Bad Request"
 
     def test_multipart_field_too_big(self):
         """
@@ -129,9 +134,10 @@ class TestParseMultipart(object):
         app = FrescoApp()
         app.route('/', POST, view)
 
-        r = Agent(app).post_multipart(data=[('f1', 'x' * 200)],
-                                          check_status=False)
-        assert r.response.status == "413 Payload Too Large"
+        with app.requestcontext_post('/',
+                                     multipart=True,
+                                     data=[('f1', 'x' * 200)]):
+            assert app.view().status == "413 Payload Too Large"
 
 
 class TestParseFormEncodedData(object):
@@ -155,8 +161,9 @@ class TestParseFormEncodedData(object):
         app = FrescoApp()
         app.route('/', POST, view)
 
-        r = Agent(app).post(data=[('f1', 'x' * 200)], check_status=False)
-        assert r.response.status == "413 Payload Too Large"
+        with app.requestcontext_post('/',
+                                     data=[('f1', 'x' * 200)]):
+            assert app.view().status == "413 Payload Too Large"
 
     def test_posted_data_contains_non_ascii_chars(self):
         """
@@ -174,7 +181,7 @@ class TestParseFormEncodedData(object):
         with app.requestcontext(
                 '/',
                 REQUEST_METHOD='POST',
-                CONTENT_LENGTH=ustring_type(len(data)),
+                CONTENT_LENGTH=ustr(len(data)),
                 CONTENT_TYPE='application/x-www-form-urlencoded',
                 wsgi_input=data):
             response = app.view()
@@ -203,23 +210,6 @@ class TestParseFormEncodedData(object):
             assert request.query['char'] == self.char
 
 
-class TestParseBody(object):
-
-    def test_formencoded_data_too_big(self):
-        """
-        POST data raises an exception if it exceeds request.MAX_SIZE bytes
-        """
-        def view():
-            request = context.request
-            request.MAX_SIZE = 100
-            request.body
-            return Response(['ok'])
-        app = FrescoApp()
-        app.route('/', POST, view)
-
-        r = Agent(app).post(data=b'x' * 200, check_status=False)
-        assert r.response.status == "413 Payload Too Large"
-
 
 class TestEncodeMultipart(object):
 
