@@ -1,4 +1,20 @@
-from flea import Agent
+# Copyright 2015 Oliver Cope
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
+#
+from __future__ import absolute_import
+
+from fresco import FrescoApp, Response
 from fresco.util.wsgi import ClosingIterator, StartResponseWrapper
 
 
@@ -69,4 +85,8 @@ class TestStartResponseWrapper(object):
             write2(b'mat')
             return []
 
-        assert Agent(wsgiapp).get('/').body == 'catsatmat'
+        with FrescoApp().requestcontext() as c:
+            r = Response.from_wsgi(wsgiapp,
+                                   c.request.environ,
+                                   lambda status, headers: None)
+            assert b''.join(r.content) == b'catsatmat'
