@@ -1,6 +1,17 @@
+# Copyright 2015 Oliver Cope
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
 #
-# Copyright (c) 2009-2014 Oliver Cope. All rights reserved.
-# See LICENSE.txt for terms of redistribution and use.
 """
 Utilities for working with data on the HTTP level
 """
@@ -156,7 +167,9 @@ ParsedContentType = namedtuple(
     'ParsedContentType', 'content_type encoding params')
 
 
-def get_content_type_info(environ, default_type=None, default_encoding=None):
+def get_content_type_info(environ,
+                          default_type='application/octet-stream',
+                          default_encoding='iso-8859-1'):
     """
     Read and parse the Content-Type header and return a
     :class:`ParsedContentType` object.
@@ -499,7 +512,7 @@ class FileUpload(object):
 def encode_multipart(data=None, files=None, charset='UTF-8', **kwargs):
     """
     Encode ``data`` using multipart/form-data encoding, returning a tuple
-    of ``(<encoded data>, <environ items>)
+    of ``(<encoded data>, <environ items>)``.
 
     :param data: POST data to be encoded, either a dict or list of
                  ``(name, value)`` tuples.
